package com.nisum.ascend.inventory.controller;

import com.nisum.ascend.inventory.dto.ProductDto;
import com.nisum.ascend.inventory.model.Product;
import com.nisum.ascend.inventory.repository.ProductRepository;
import com.nisum.ascend.inventory.service.ProductService;
import lombok.extern.slf4j.Slf4j;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.reactive.AutoConfigureWebTestClient;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.ResponseEntity;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.junit.jupiter.SpringExtension;
import org.springframework.test.web.reactive.server.WebTestClient;
import org.springframework.web.bind.annotation.ResponseBody;
import reactor.core.publisher.Mono;

import static org.junit.jupiter.api.Assertions.*;

@ExtendWith(SpringExtension.class) //integrates springtestframework into junit 5
@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT) //provides support for testing springboot app
@DirtiesContext
@AutoConfigureWebTestClient //autoconfigures the webclient
@ActiveProfiles("test")
@Slf4j
class ProductControllerTest {

    @Autowired
    private WebTestClient client;


    @BeforeEach
    public  void setup(){

    }

    @Test
    void testProductInvalidSkuNotFound() {
        client
                .get()
                .uri("/products/".concat("{sku}"), "invalid")
                .exchange()
                .expectStatus()
                .isNotFound();
    }

    @Test
    void testProductBySkuFound() {
        String sku = "100000";

        client.get()
                .uri("/products/".concat("/{sku}"), sku)
                .exchange()
                .expectStatus()
                .isOk()
                .expectBody()
                .jsonPath("$.sku", sku);
    }
}